"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs-extra");
var cp = require("child_process");
var tar = require('tar');
var child = null;
var updateCheckTimer = null;
function restartApp() {
    child.kill();
    if (fs.existsSync("update.tar.gz")) {
        tar.x({
            file: 'update.tar.gz'
        }).then(function (_) {
            fs.unlinkSync("update.tar.gz");
            if (process.env.NODE_ENV == 'production') {
                fs.copy("./opt/netonix-manager", ".", function (err) {
                    fs.removeSync("./opt");
                    startApp();
                });
            }
            else {
                // this is for testing
                fs.removeSync("./opt");
                startApp();
            }
        });
    }
    else {
        startApp();
    }
}
function startApp() {
    var crashlog = "";
    console.log("starting child process");
    var exe = "node";
    // on some systems the node executable is nodejs
    if (fs.existsSync("/usr/bin/nodejs")) {
        exe = "nodejs";
    }
    var ver_test = cp.spawn(exe, ["--help"]);
    var help_results = "";
    ver_test.stdout.setEncoding('utf8');
    ver_test.stdout.on('data', function (data) {
        help_results += data;
    });
    ver_test.on('exit', function () {
        var tls_min_enforced = help_results.search('--tls-min-v1.0');
        if (tls_min_enforced > 0) {
            child = cp.spawn(exe, ['--tls-min-v1.0', 'app.js']);
        }
        else {
            child = cp.spawn(exe, ['app.js']);
        }
        child.stdout.setEncoding('utf8');
        child.stdout.on('data', function (data) {
            console.log(data.toString().trim());
        });
        child.stderr.on('data', function (data) {
            crashlog += data;
        });
        child.on('close', function (code) {
            console.log('process exit code ' + code);
            // if the app unexpectedly exits then restart it
            if (!fs.existsSync("update.tar.gz")) {
                fs.appendFileSync("crash.log", crashlog);
                clearInterval(updateCheckTimer);
                startApp();
            }
        });
        updateCheckTimer = setInterval(function () {
            if (fs.existsSync("update.tar.gz")) {
                clearInterval(updateCheckTimer);
                restartApp();
            }
        }, 5000);
    });
}
process.on('SIGTERM', function () {
    child.kill();
    process.exit(0);
});
startApp();
//# sourceMappingURL=bootloader.js.map