(function ($) {
	var scrollBarWidth = 15;

	$.fn.scrollTableBody = function (options) {
		var table = this;
		var parent;
		if (options && options.parent) {
			parent = options.parent;
		} else {
			parent = table.parent();
		}

		wrapTable(table, parent);
		alignColumns(table);

		var resizeAlignFunction = function () { alignOnResize(table, parent); };
		$(window).resize(resizeAlignFunction);
	};

	function wrapTable(table, parent) {
		var existingClasses = table.attr('class');
		var existingMarginBottom = table.css('margin-bottom');
		table.css('margin-bottom', 0);
		var rowHeight = table.find('tbody tr:first').outerHeight();
		var offset = table.attr("data-scroll-table-offset");
		if (typeof offset === typeof undefined || offset === false) {
			offset = 0;
		}

		var tableHeight = parent.innerHeight() - rowHeight - 10 - offset;

		var headerTable = $('<table style="table-layout:fixed;width:auto;margin-bottom:0;" class="jqstb-header-table ' + existingClasses + '"><thead><tr><td></td></tr></thead></table>'),
			scrollDiv = '<div class="jqstb-scroll" style="height:' + tableHeight + 'px;overflow-y:auto"></div>';

		// Insert the table that will hold the fixed header, and insert the div that will get scrolled
		table.before(headerTable).before(scrollDiv);
	}

	function alignColumns(table) {
		table.each(function (index) {
			// To minimize "Flash of Unstyled Content" (FOUC), set the relevant variables before manipulating the DOM
			var dataTable = $(this);
			var headerTable = dataTable.prev().prev();

			// Place main table data inside of relevant scrollable div (using jQuery eq() and index)
			var scrollDiv = headerTable.next();
			scrollDiv.prepend(dataTable);
			var scrollEl = scrollDiv[0];

			var hasHorizontalScroll = scrollEl.clientWidth < scrollEl.scrollWidth;
			//scrollDiv.outerWidth(fixedTableWidth + scrollBarWidth + 2);

			if (hasHorizontalScroll) {
				var dataTableWidth = dataTable.outerWidth();
				headerTable.width(dataTableWidth);
				scrollDiv.outerWidth(scrollEl.clientWidth);
				var scrollDivWidth = scrollDiv.outerWidth();

				var width = scrollDivWidth - scrollBarWidth;

				var headerScrollDiv = $('<div style="overflow:hidden;width:' + width + 'px" class="jqstb-header-scroll"></div>');
				headerTable.wrap(headerScrollDiv);

				scrollDiv.on('scroll', function () {
					$('div.jqstb-header-scroll').scrollLeft($(this).scrollLeft());
				});
			}

			// Insert header data into fixed header table
			headerTable.find('thead').replaceWith(dataTable.children('caption, thead').clone());

			// Force column widths to be set for each header column
			var srcCols = dataTable.find('thead tr:first th');
			var dstCols = headerTable.find('thead tr:first th');
			var dstCols2 = dataTable.find('tbody tr:first td');
			for (i = 0; i < srcCols.length; i++) {
				dstCols.eq(i).outerWidth(srcCols.eq(i).outerWidth());
				dstCols2.eq(i).outerWidth(srcCols.eq(i).outerWidth());
			}

			// Hide original caption, header
			dataTable.children('caption, thead').hide();
		});
	}

	function alignOnResize(table, parent) {
		table.each(function (index) {
			var dataTable = $(this);
			var rowHeight = dataTable.find('tbody tr:first').outerHeight();
			if (rowHeight > 0) {
				var headerTable = dataTable.parent().prev();

				// Temporarily show the inner table's header since the dom calculates width based on them being visible
				dataTable.children('thead').show();
				if (dataTable.find('tbody tr').length > 1) {
					// showing the header increases the height of the table so we hide the first row to compensate
					// this doesn't account if the header isn't the same height as the first row
					// but its enough to get the VLANs table working
					dataTable.find('tbody tr:first').hide();
				}

				var scrollEl = headerTable.next();
				var hasHorizontalScroll = scrollEl[0].clientWidth < scrollEl[0].scrollWidth;
				if (hasHorizontalScroll) {
					var scrollDivWidth = $('div.jqstb-scroll').outerWidth();
					$('div.jqstb-header-scroll').outerWidth(scrollDivWidth - scrollBarWidth);
				}

				// Force column widths to be set for each header column
				var srcCols = dataTable.find('thead tr:first th');
				var dstCols = headerTable.find('thead tr:first th');
				var dstCols2 = dataTable.find('tbody tr:first td');
				for (i = 0; i < srcCols.length; i++) {
					dstCols.eq(i).outerWidth(srcCols.eq(i).outerWidth());
					dstCols2.eq(i).outerWidth(srcCols.eq(i).outerWidth());
				}

				// Hide the inner table's header when we're done 
				dataTable.children('thead').hide();
				dataTable.find('tbody tr:first').show();

				var offset = dataTable.attr("data-scroll-table-offset");
				if (typeof offset === typeof undefined || offset === false) {
					offset = 0;
				}
				var tableHeight = parent.innerHeight() - rowHeight - 10 - offset;
				scrollEl.css('height', tableHeight + 'px');
			}
		});
	}
})(jQuery);
